/* 
 * File:   LCD.h
 */

#ifndef LCD_H
#define	LCD_H

#define _XTAL_FREQ 20000000
/******************************************************************************
 *Die folgenden Definitionen und Variablen mssen an die jeweilige Pin-Belegung
 *angepasst werden
 ******************************************************************************/
#define LCD_E  RC7 //Enable LCD
#define LCD_RW RC6 //Read Write LCD
#define LCD_RS RC5 //Befehl/Datum LCD
#define LCD_Bus PORTB //PORT der an den LCD-Bus angeschlossen ist
#define TRIS_LCD TRISB //Tristate-Port-Einstellung fr LCD_Bus

char PORT_Def_in = 0b00111100; //Definition der benutzten Pins fr LCD_Bus
char PORT_Def_out = 0b11000011; //Zum lschen der benutzten Pins
char LCD_Busy = 0b00100000; //Definition des Ortes von DB7 am PIC-Port
char DB5_4 = 0b00001100; //DB5..4-Bits am PORT, fr LCD_init
char DB5 = 0b00001000; //DB5-Bit am PORT, fr LCD_init
//
void Senden_an_LCD (void); //Datum oder Befehl an Lcd senden
//Die oberen 4 Bit des Wertes an LCD bertragen
void Obere_4_Bit (char Wert) {
    LCD_Bus &= PORT_Def_out;//LCD_Bus-Pins lschen, kann vielleicht weg
    LCD_Bus |= (Wert & 0b11110000)>>2; //Die oberen 4 Bit von Wert auf den Bus
    Senden_an_LCD ();
}

//Die unteren 4 Bit des Wertes an LCD bertragen
void Untere_4_Bit (char Wert) {
    LCD_Bus &= PORT_Def_out;//LCD_Bus-Pins lschen
    LCD_Bus |= (Wert & 0b00001111)<<2; //Die unteren 4 Bit von Wert auf den Bus
    Senden_an_LCD ();
}
//char PORT_Def_in = 0b11110000; //Definition der benutzten Pins fr LCD_Bus
//char PORT_Def_out = 0b00001111; //Zum lschen der benutzten Pins
//char LCD_Busy = 0b10000000; //Definition des Ortes von DB7 am PIC-Port
//char DB5_4 = 0b00110000; //DB5..4-Bits am PORT, fr LCD_init
//char DB5 = 0b00100000; //DB5-Bit am PORT, fr LCD_init

//void Senden_an_LCD (void); //Datum oder Befehl an Lcd senden
////Die oberen 4 Bit des Wertes an LCD bertragen
//void Obere_4_Bit (char Wert) {
//    LCD_Bus &= PORT_Def_out;//LCD_Bus-Pins lschen, kann vielleicht weg
//    LCD_Bus |= (Wert & 0b11110000); //Die oberen 4 Bit von Wert auf den Bus
//    Senden_an_LCD ();
//}
//
////Die unteren 4 Bit des Wertes an LCD bertragen
//void Untere_4_Bit (char Wert) {
//    LCD_Bus &= PORT_Def_out;//LCD_Bus-Pins lschen
//    LCD_Bus |= (Wert & 0b00001111)<<4; //Die unteren 4 Bit von Wert auf den Bus
//    Senden_an_LCD ();
//}
/**Ende Anpassungen***********************************************************/

/***Befehle an das LCD********************************************************/
/*Display lschen 0b00000001 lschen & Home
//Cursor z.Anfang 0b00000010 Home, kein Display-Shift
//Entry Mode Set  0b000001.I/D.S  ;I/D: Cursor Laufrichtung, S: Shift ein/aus
//Display ein/aus 0b00001.D.C.B ;D: Display ein/aus C: Cursor ein/aus
                                //B: Blinkender Cursor ein/aus
//Curs./Dis.-Shift 0b0001.S/C.R/L.xx ;S/C: Display oder Cursor,
                                //R/L: nach rechts oder links schieben
//Function Set    0b001.DL.N.F.xx ;DL: Interface 8-/4-Bit,N: ein-/zweizeilig
                                //F:5x8/5x11 Zeichen-Matrix
//CG RAM Addr. Set 0b01xxxxxx ;CG RAM-Adresse einstellen
//DD RAM Addr. Set 0b1xxxxxxx ;DD RAM-Adresse einstellen, Display
//              Cursorposition,0..0x28 Zeile 1, 0x40..0x68 Zeile 2, also je 40
//              Zeichen. Beispiel: Befehl_an_LCD(0x80+0x0A)=Zeile 1, Pos.10
//              Es sind jeweils nur 16 Zeichen sichtbar
//Beispiele:
//0b00000001    lschen & Home
//0b00101000    4bit & 2 zeilig
//0b00001000    Display aus
//0b00000110    Automatisches Weiterspringen
//0b0001abnn    schieben:a=1 Display, a=0 Cursor, b=1 rechts, b=0 links, n=egal
/***Ende Befehle an das LCD***************************************************/

//Funktionen*/
void LCD_init (void);   //LCD initialisieren, 4bit & 2 zeilig, automatisches
                        //weiterspringen, Cursor Home, Display an
void Daten_an_LCD (char);  //Daten auf LCD bertragen
void Befehl_an_LCD(char);  //Befehle auf LCD bertragen
void Warten_auf_LCD (void); // Warten, bis LCD wieder empfangsbereit
void Zahl_an_LCD (char, char); //(Wert, Tabulator) Gibt Pos. Dezimalzahl mit vierer Tabulator aus
void LCD_String (const char *pString); //String an LCD ausgeben, Format: "String"

//Ausgabe der Zahl an das LCD im ASCII-Code
//Negative Werte gehen (noch) nicht
void Zahl_an_LCD (char Zahl,char i) { //i*4 ist der Tabulator
    Befehl_an_LCD(0x80+i*4); //LCD-Adresse, 1. Zeile jede vierte Stelle
                    //Befehl(0x80) = Cursoradresse
    char einer = Zahl%10; //Einerstelle des Wertes, Modulo 10
    char zehner = Zahl%100/10; //Zehnerstelle, Modulo 100
    char hunderter = Zahl/100; //Hunderter
    if (hunderter == 0) {
        Daten_an_LCD (' '); //Ausgabe rechtsbndig
        if (zehner == 0) Daten_an_LCD (' '); //Ausgabe rechtsbndig
        else Daten_an_LCD (zehner+48); //ASCII (48) = 0
    }
    else {
        Daten_an_LCD (hunderter+48); //ASCII (48) = 0
        Daten_an_LCD (zehner+48); //ASCII (48) = 0
    }
    Daten_an_LCD (einer+48); //Einer immer ausgeben auch wenn Null
}

//Befehl an LCD bertragen
void Befehl_an_LCD (char Wert){
    Warten_auf_LCD();
    LCD_RS = 0; //Befehl = 0, Datum =1
    //Jetzt erst die oberen und dann die unteren Bits senden
    Obere_4_Bit (Wert);
    Untere_4_Bit (Wert);
}

//Daten an LCD bertragen
void Daten_an_LCD (char Wert){
    Warten_auf_LCD();
    LCD_RS = 1;    //Befehl = 0, Datum = 1
    //Jetzt erst die oberen und dann die unteren Bits senden
    Obere_4_Bit (Wert);
    Untere_4_Bit (Wert);
}

//Senden an LCD
void Senden_an_LCD (void) {
    LCD_E = 1;
    NOP();
    LCD_E = 0;
}

//Warten bis DB7 = Busy-Flg des LCD wieder auf null steht
void Warten_auf_LCD (void) {// Warten, bis LCD wieder empfangsbereit
    char PORT_Back = 0;    //LCD_Bus-PORT Backup
    TRIS_LCD |= PORT_Def_in;    //LCD_Bus-Pins auf IN
    do { //Busy-Flag abfragen
        LCD_RS = 0;    //Befehl = 0, Datum = 1
        LCD_RW = 1;    //LCD auf Lesen stellen
        LCD_E = 1;    //Senden/Empfangen
        NOP(); //Sicherheitswarten
        PORT_Back = LCD_Bus; //Brauchts fr die Abfrage, ob DB7 HI
        LCD_E = 0;    //Senden/Empfangen beenden
        NOP(); //Sicherheitswarten
        Senden_an_LCD (); //Ich wei nicht wofr! Aber ohne gehts nicht!
    }
    while(PORT_Back & LCD_Busy); //Ist das LCD noch beschftigt?
    TRIS_LCD &= PORT_Def_out;    //LCD_Bus-Pins auf OUT
    LCD_RW = 0;    //Wieder auf schreiben stellen
}

//String an LCD bertragen
//Aufruf: LCD_String("String")
void LCD_String (const char *pString){
    char zeichen;
    zeichen = *pString; // Den Inhalt des Registers, auf das der Zeiger
    //deutet,in die Variable "zeichen" kopieren.
    while(zeichen > 0){ //String ist Null-Terminiert, also letztes Zeichen ist 0
        Daten_an_LCD(zeichen); // Zeichen am LC-Display ausgeben
        pString++; // Zeiger um 1 erhoehen, er zeigt nun auf das
        // naechste Zeichen der auszugebenden Zeichenkette
        zeichen = *pString; //
    }   
}

//LCD initialisieren
void LCD_init (void){
    __delay_ms(125);// Zu Beginn 125ms warten
    for (char i = 0; i<3; i++) { // Schleife 3 mal durchlaufen
        LCD_Bus &= PORT_Def_out; //LCD_Bus lschen
        LCD_Bus |= PORT_Def_in & DB5_4; //Vorgegebener Wert
        Senden_an_LCD ();
        __delay_ms(10);  //10ms Delay
    }
    LCD_Bus &= PORT_Def_out; //LCD_Bus lschen
    LCD_Bus |= (PORT_Def_in & DB5); //Vorgegebener Wert
    Senden_an_LCD ();
    __delay_ms(10);  //10ms warten

    Befehl_an_LCD(0b00000001); //lschen & Home
    Befehl_an_LCD(0b00101000); //4bit & 2 zeilig
    Befehl_an_LCD(0b00001000); //Display aus
    Befehl_an_LCD(0b00000110); //Automatisches Weiterspringen
    Befehl_an_LCD(0b00000011); //Cursor Home
    Befehl_an_LCD(0b00001100); //Display an, kein Cursor
}


#endif	/* LCD_H */

