/* 
 * File:   Slave_1519_2.c
 * Version mit LCD und I2C
 */

#include <stdio.h>
#include <stdlib.h>
#include <xc.h>
#include "Init.h"
#include "LCD.h"

#pragma config FOSC = HS        // Oscillator Selection bits (HS oscillator)
#pragma config WDTE = OFF       // Watchdog Timer Enable bit (WDT disabled)
#pragma config PWRTE = ON       // Power-up Timer Enable bit (PWRT enabled)
#pragma config BOREN = OFF      // Brown-out Reset Enable bit (BOR disabled)
#pragma config LVP = OFF        // Low-Voltage (Single-Supply) In-Circuit Serial Programming Enable bit (RB3 is digital I/O, HV on MCLR must be used for programming)
#pragma config CPD = OFF        // Data EEPROM Memory Code Protection bit (Data EEPROM code protection off)
#pragma config WRT = OFF        // Flash Program Memory Write Enable bits (Write protection off; all program memory may be written to by EECON control)
#pragma config CP = OFF         // Flash Program Memory Code Protection bit (Code protection off)

#define _XTAL_FREQ 20000000 //Wichtig fr __delay_...
#define Status RA5 //Status-LED an Pin RA5
#define Taster RA4 //Taster N.H.
__eeprom char a[1];

//Variabeln
//Tabulator fr 2-zeilige-LCDs
char Tabulator [8] = {  0,  1,  2,  3,
                        16, 17, 18, 19}; //LCD ist in 4 Tabulatorstellen unterteilt
//Tabulator fr 4-zeilige-LCDs
//char Tabulator [20] = {  0,  1,  2,  3,  4,
//                        16, 17, 18, 19, 20,
//                         5,  6,  7,  8,  9,
//                        21, 22, 23, 24, 25}; //LCD ist in 4 Tabulatorstellen unterteilt
char Slave_Wert[2]; //Werte die der Slave an den Master bergeben soll
char Registervommaster; //is klar
char Wertvommaster[16]; //dito
char junk; //Brauchts fr das Auslesen des Puffers
char T0ck; //Zhler berlufe TMR0
bit I2C_Read; //Der Master liest gerade 
bit I2C_Write; //Der Master schreibt gerade
char I2C_Count; //Einzelschritte beim I2C
bit Master_ruft; //Vom Master Datum erhalten = 1
char Z; //Zhlvariabel

//Funktionen
void init(void); //Chip initialisieren, incl. I2C
void interrupt Tmr0_SSP (void);

/*
 * 
 */
void main(void) {
    init(); //Chip einstellen
    LCD_init(); //LCD starten und einstellen
    //Flags lschen:
    I2C_Read = 0; //Der Master liest nicht
    I2C_Write = 0; //Der Master schreibt nicht
    //Interrupts ein/ausschalten
/***Jetzt bernehmen wir Daten vom Master**************************************/
    PEIE = 1; //Periphel Interrupt en/disable
/******************************************************************************/
    LCD_String("Bereit"); //Bereit steht auf dem LCD
    __delay_ms(500); //eine halbe Sekunde lang
    Befehl_an_LCD(0b00000001); //lschen & 
    while(1) { //Arbeitsschleife
        while (!Master_ruft); //Warten, bis Master ein Datum geschickt hat
        Master_ruft = 0; //Flag zurcksetzen
        //Wertvommaster an entsprechender Stelle auf dem LCD anzeigen
        Zahl_an_LCD(Wertvommaster[Registervommaster], Tabulator [Registervommaster]);
    }
}

//Interrupt bei berlauf des Timer 0 und bei SSP-event
void interrupt Tmr0_SSP (void){ //Name dient der Orientierung, ist egal was da steht
    if (TMR0IF) { //TMR0 ist bergelaufen
        T0ck++;//Status-LED blinkt -> PIC arbeitet "korrekt" 
        if (T0ck == 22) { //Prescaler 256  *256*22 = 1441792Cyc. / 5 ~ 300ms
            Status = !Status;//toggeln
            T0ck = 0; //Modulo 22
        }
        TMR0IF = 0; //Flag lschen, sonst bleibt das Programm im Interrupt hngen
    }
//Wenn der Master lesen will, schickt er erst einen Wert, der das zu bermittelnde
//Byte spezifiziert. Wenn der Master etwas sendet, dann sind das immer zwei Bytes
//erst die Spezifikation des Datums, dann das Datum.
    if (SSPIF){//Ist in SSPBUF etwas angekommen?
        SSPIF = 0; //Flag lschen, sonst...
            //erstes und evtl. drittes SSP-Interrupt
            if(!D_nA) { //Ist eine Adresse bermittelt worden?
                junk = SSPBUF;//Puffer auslesen, dummy read to clear the BF bit
                if (R_nW) { //Master will lesen, LSB des Adressbytes = 0
                    I2C_Read = 1; //Master will lesen
                    I2C_Write = 0; //Es wird nicht geschrieben
                    I2C_Count = 0; //Schrittzhler
                    //Nach einer Leseaufforderung, wird das zu bermittelnde
                    //Byte sofort auf den Bus gelegt, Kein SSP-Interrupt dazwischen
/*******************Test***********************************************************/
                    SSPBUF = Taster;
/**********************************************************************************/
                    //SSPBUF = Slave_Wert[Registervommaster]; //Zu bermittelnden Wert in den Puffer schieben
                    CKP = 1; //Release Clock Stretch
                //Nach der bertragung hlt der Slave die CLK-Leitung auf GND (LO)
                //release clock stretch, Master darf weiter schreiben
                    Master_ruft = 1; //Ein Schreib/Lesevorgang ist abgeschlossen
                }
                else { //Master will schreiben, LSB des Adressbytes = 1
                    I2C_Read = 0; //Master will nicht lesen
                    I2C_Write = 1; //Es wird geschrieben
                    I2C_Count = 0; //Schrittzhler
/*Testen, warum hier CKP auskommentiert ist*************************************/
//Weil das CKP unten am Schluss von SSP-Interrupt zu viel ist: Also dieses rein
//und das unten raus!!!
                    //CKP = 1; //Release Clock, mit geht es auf jeden Fall nicht!!!
/*******************************************************************************/
                }
            }
            //zweites und evtl. drittes Interrupt
            else { //=D_nA = 1, ein Datum ist bermittelt worden
                if(I2C_Write) { //Master will schreiben
                    switch(I2C_Count) {
                        case 0: //Erster Interrupt nach Adresse + schreiben
                            Registervommaster = SSPBUF; //bertragenes Byte speichern, BF lschen
                            I2C_Count++; //Interruptzhler
                            CKP = 1; //Bus frei geben
                            break; //Die Funktion switch() verlassen
                        case 1: //Zweiter Interrupt nach Adresse + schreiben 
                            Wertvommaster[Registervommaster] = SSPBUF; //bertragenes Byte speichern
                            I2C_Count++; //Interruptzhler
                            I2C_Write = 0; //Schreibvorgang ist beendet
                            CKP = 1; //Bus frei geben
                            Master_ruft = 1; //Ein Schreib/Lesevorgang ist abgeschlossen
                            break; //Die Funktion switch() verlassen
                        default : //Hierher kommt das Programm nicht, ausser...
                            break; //Die Funktion switch() verlassen
                    }
                }
                if(I2C_Read) { //Master will lesen
                    //Sollen mehrere Daten gelesen werden, muss hier die 
                    //entsprechende Routine hin. 
                    //Brauchen wir aber m.E. nicht.
                    CKP = 1; //Bus frei geben
                    I2C_Read = 0;
                }
            }
/*Testen, ob das CKP zuviel ist************************************************/
//Ja das ist zuviel, wenn das raus kommt, kann es an der richtigen Stelle (s.o.) raus
            CKP = 1;
/******************************************************************************/
    }
}