#include <stdio.h>
#include <stdlib.h>
#include <xc.h>
#include "Init.h"
#include "MLX_read.h"
//#include "MLX_Neu.h" //Muss aktiviert werden, wenn der T-Sensor eine neue Adresse bekommen soll (s.u.))

// CONFIG1
#pragma config FOSC = HS        // Oscillator Selection (HS Oscillator, High-speed crystal/resonator connected between OSC1 and OSC2 pins)
#pragma config WDTE = OFF       // Watchdog Timer Enable (WDT disabled)
#pragma config PWRTE = ON       // Power-up Timer Enable (PWRT enabled)
#pragma config MCLRE = ON       // MCLR Pin Function Select (MCLR/VPP pin function is MCLR)
#pragma config CP = OFF         // Flash Program Memory Code Protection (Program memory code protection is disabled)
#pragma config BOREN = OFF      // Brown-out Reset Enable (Brown-out Reset disabled)
#pragma config CLKOUTEN = OFF   // Clock Out Enable (CLKOUT function is disabled. I/O or oscillator function on the CLKOUT pin)
#pragma config IESO = OFF       // Internal/External Switchover (Internal/External Switchover mode is disabled)
#pragma config FCMEN = OFF      // Fail-Safe Clock Monitor Enable (Fail-Safe Clock Monitor is disabled)

// CONFIG2
#pragma config WRT = OFF        // Flash Memory Self-Write Protection (Write protection off)
#pragma config VCAPEN = OFF     // Voltage Regulator Capacitor Enable bit (VCAP pin function disabled)
#pragma config STVREN = OFF     // Stack Overflow/Underflow Reset Enable (Stack Overflow or Underflow will not cause a Reset)
#pragma config BORV = LO        // Brown-out Reset Voltage Selection (Brown-out Reset Voltage (Vbor), low trip point selected.)
#pragma config LPBOR = OFF      // Low-Power Brown Out Reset (Low-Power BOR is disabled)
#pragma config LVP = OFF        // Low-Voltage Programming Enable (High-voltage on MCLR/VPP must be used for programming)

#define _XTAL_FREQ 20000000
//LATX ist ein Register, das die Ausgabe des zugehrigen PORTX regelt.
//PORTX ist der Zustand an den Pins
//LATX kann also anders sein als PORTX
#define Status LATA4 //Bit 4 des Registers LATA(=PORTA), 0 LED ein, 1 LED aus
//#define vorwaerts LATD = LATD & 0b11110000 | 0b00001010 //Beim Inhalt des Registers
//LATD werden zunchst die unteren 4 Bits gelscht und dann Bit 1 und 3 gesetzt.
//An PORTD liegt also dann an Pin 22 und 24 fnf Volt an, an Pin RD7..4 liegt der
//Pegel an, der auch vorher angelegen hat
//#define stopp LATD = LATD & 0b11110000
#define vorwaerts   LATD = (LATD & 0b11110000)|0b00001010
#define links       LATD = (LATD & 0b11110000)|0b00000110
#define rechts      LATD = (LATD & 0b11110000)|0b00001001
#define stopp       LATD = (LATD & 0b11110000) 
//Definitionen sinnvoll ergnzen: rueckwaerts, rechts_drehen....
#define US_Trigger LATD5 //US-Trigger Pin an RD5, Pin 28
#define pwm_links   CCPR1L 
#define pwm_rechts  CCPR2L

//Deklaration der Variabeln, mit Wertzuweisung
const char PIC_Slave = 0x14; //PIC-Slave Adresse vom 16F876A
//Es stehen 512 Speicherpltze zur Verfgung
//Jeweils 80 Bytes pro Bank, 31 Bnke
//Diese sind in den Speicherpltzen 0x2000 - 0x29AF gespiegelt
//Liegen also hintereinander, was eine indirekte Adressierung vereinfacht

//Deklaration der Variabeln, ohne Wertzuweisung
char A_D_Wert [28]; //28 A/D-Wandler-Pins am 1519
char Channel; //Angewhlter AD-Kanal

char I2C_Add; //aktuell zu verarbeitende I2C-Adresse
char I2C_Index_w; //Write Index, aktuell zu verarbeitendes Register/Datum-Paar
char I2C_Index_R; //Read Index, aktuell zu verarbeitendes Register/Datum-Paar
bit I2C_Write; //Ist der I2C-Bus gerade mit Schreiben beschftigt?
bit I2C_Read; //Ist der I2C-Bus gerade mit Lesen beschftigt?
char I2C_Count; //Zhler der Schritte bei der I2C-Kommunikation
char Count; //Zhler fr I2C_Index_w

char Register_S [8] = {0, 1, 2, 3, 4, 5, 6, 7}; //LCD Tabulatorstelle
char Datum_S[8]; //ans LCD zu bertragene Werte
char Datum_R; //vom Slave gelesener Wert
int Tick; //Zhler der TMR0 berlufe im Interrupt
char m, n; //Zhler im Interrupt
char n_US; //Zhler fr US-Trigger, 40ms
char Zeit_US; //Laufzeit des US-Signals
char Abstand_US; //Abstand vom US in cm (ca. 10% ungenau)
bit Flag_LED; //Soll die LED blinken (1) oder an sein (0)

bit MLX_Slave_toggel;
char I2C_Add;
char MLX_Slave = 0x5A;
char Register_R[2] = {0x07, 0x17};
char Temperatur_LO_rechts;
char Temperatur_LO_links;
char Temperatur_HI_rechts;
char Temperatur_HI_links;

char Encoder;
int Strecke; //Lnge des zu fahrenden Wegs
bit Flag_Strecke_erreicht = 0; //die vorgegebene Strecke ist erreicht
bit Flag_Strecke_messen = 0; //Es wird gerade eine Strecke gemesen


void init(void); //Chip initialisieren
void Strecke_fahren(char, char); //(Lnge, Richtung)
void Strecke_30cm(void);
void links_drehen(void);
void rechts_drehen(void);
void interrupt Interrupt(void); //sic
/*
 * Jetzt gehts los
 */

/*Hauptfunktion*/
void main(void) {
    init(); //Chip initialisieren, incl. I2C
    
//    MLX_Neu(); //Dem Temperatursensor eine neue Adresse geben
    //Dazu mssen die Zeile 6 (#include "MLX_Neu.h")  und die Zeile hier drber
    //aktiviert werden. Kurz, 1s, den Roboter einschalten und dann alles wieder
    //auskommentieren. Es darf dabei natrlich nur ein T-Sensor angeschlossen sein.
    
    //Zum Debuggen knnen Interrupts ein und aus geschaltet werden:
    TMR1IE = 0; //En/Disable TMR1-Interrupt
    IOCIE = 1; //En/Disable Interupt on RB7..1/0
    TMR0IE = 1; //En/Disable TMR0 Interrupt
    SSPIE = 1; //En/Disable SSP Interrupt
    INTE = 0; //En/Disable Interrupt on RB0
    ADIE = 0; //En/Disable Interrupt on AD-Module
    //Flags zurck setzen
    I2C_Write = 0; //Ist der I2C gerade mit Schreiben beschftigt?
    I2C_Read = 0; //Ist der I2C gerade mit Lesen beschftigt?
    Flag_LED = 1;
    __delay_ms(500); //Allen Sensoren Zeit lassen auf den Anfangswert einzupegeln
    GIE = 1; //En/Disable General Interrupt
    //Fr Tests:
    //char Z = 0;
    //Datum_S[0] = 123;
    while (1) { //Endlosschleife
        
/****************************************************************************
 * im Interrupt in IOCBF5 in else ergnzen TMR1IF = 0;
 *****************************************************************************/
        if (!Flag_Strecke_messen) {
            Strecke_30cm();
        }
        if (Flag_Strecke_erreicht) {
            stopp;
            __delay_ms(100);
            Flag_Strecke_messen = 0;
        }
    }
}

void Strecke_fahren(char Length, char Direction) {
    IOCIE = 0; //Jetzt knnen wir keinen Interrupt on change gebrauche
    /*Encoder Integer?************************************************/
    Encoder = 0; //Encoderzhler lschen
    /*****************************************************************/
    Strecke = Length; //Strecke an "Interrupt" bergeben, 6 Ticks/cm (90 Ticks/Umdrehung)
    Flag_Strecke_erreicht = 0; //
    Flag_Strecke_messen = 1; //
    IOCIE = 1; //Interrupt wieder an
    Direction; //Bit 3 - 0 Motorsteuerung
}

void links_drehen(void) { //90 links drehen
    Strecke_fahren(90, links); //130 Ticks des Encoders
}

void rechts_drehen(void) { //90 rechts drehen
    Strecke_fahren(90, rechts); //130 Ticks des Encoders
}

void Strecke_30cm(void) { //30cm geradeaus fahren
    Strecke_fahren(165, vorwaerts); //6 Ticks/cm (90 Ticks/Umdrehung)
}

//Interrupt
void interrupt Interrupt(void) { //sic
//    Status = 1; //Hilfreich zum debuggen
    //Der Quarz schwingt mit 20MHz, jeder Befehl braucht 4 Schwingungen
    //Alle 200ns wird ein Befehl ausgefhrt (Zyklus) = 5 MHz
    //Prescaler 1:2 -> Interrupt alle 0,102ms = 102us = 512 Zyklen = 512 Befehle
    if (TMR0IF) {
        //Da mind. 100us vorbei sind hat mittlerweile die AD-Wandlung stattgefunden (s.u.))
        A_D_Wert [Channel] = ADRESH; //A/D Resolution Hi-Byte, fr die Auswertung im Programm
/*********************************************************************/
        Datum_S[Channel] = A_D_Wert[Channel]; //HI-Teil des AD-Registers reicht vllig
/*********************************************************************/
        //Analogwert initialisieren, alle 102us
        Channel++; //AD-Kanal, nxter Kanal
        //nxte Zeile: n anpassen an die Anzahl der AD-Kanle
        if (Channel == 4) Channel = 0; //Maze braucht 4, erstmal, Line braucht 8
        ADCON0 = 0b10000001 | (Channel << 2); //Bit 6-2 whlen den Channel aus
/*******Jetzt mssen mind. 5us = 25 Zyklen vergehen, bevor die Wandlung gestartet werden darf
 *******D.H, wenn ab hier was gestrichen wird muss man das testen!!!!!!!!!!!!!!!!!!!!!!!!!!*/
//
        Tick++; //Blinken der Status-LED, PIC arbeitet "korrekt"
        if(Flag_LED) { //Soll die LED blinken?
            if (Tick >= 2500) { //2500 * 102us ~ 255ms
                Tick = 0; //Tick wieder auf 0 setzen
                Status = !Status; //0 ein, 1 aus
            }
        }
        else { //oder soll sie dauernd an sein?
            Status = 0; //LED dauernd ein
        }
/*Zeitstruktur fr den I2C Bus:
 * zwei Daten schreiben dauert ca. 300us, ein Datum lesen ca. 250us
 * Alle 100us wird m inkrementiert
 * Alle 10ms wird n inkrementiert
 * Alle 10ms wird ein T-Wert geholt, im Wechsel links/rechts
 * Alle 20ms wird ans LCD ein Wert bertragen, 3,3ms versetzt zu T-Werte holen
 * Alle 20ms wird ein Buchstabenwert vom Sensor geholt, 6,6ms versetzt zu T-Werte holen
 * _| |___________| |___________| |__________| |__________| |___ T-Werte holen
 * _______|||_________________________|||_______________________ LCD-Werte senden
 * ________________________| |________________________| |_______ Buchstaben holen
 * Da wre noch Platz fr einige andere I2C Module
 */
        m++; //Alle 100us wird m inkrementiert
        if (m == 100) { //Temperatur von MLX holen, ca. alle 10ms
        //Das auslesen des T-Sensors darf nicht unterbrochen werden, daher am Stck
            m = 0; //Wieder von vorne zhlen
            n_US++; //Alle 40ms US triggern
            if (n_US == 4) { //40ms um?
                n_US = 0; //Zhler wieder auf Null setzen
                US_Trigger = 1; //An Pin28 5V anlegen
                __delay_us(10); //10us warten
                US_Trigger = 0; //Trigger-Pin wieder lschen
                TMR1 = 0; //Zeitmessung beginnen
                TMR1IF = 0; //Timer-1-Interrupt-Flag lschen, max. ca. 2m messbar
            }
            n++; //fr Hufigkeit der bertragung der Werte ans LCD
            if (n == 2) n = 0;
/**********************************************************************/            
            MLX_read (); //Dauert ca. 1ms, darf nicht unterbrochen werden
/**********************************************************************/ 
        }
        if (m == 33) { //Zeitlich versetzt zu MLX_read() ausfhren
            if (n == 0) { //Werte an Slave (LCD) bertragen, alle 20ms im Wechsel mit T holen
                SSPADD = 4; //1MHz IC-Frequenz, 20MHz/(4*(4+1))=1MHz (PICTime)
                I2C_Count = 0; //Schrittzhler zurcksetzen
                I2C_Write = 1; //Jetzt wird geschrieben
                I2C_Read = 0; //Es wird nicht gelesen
                I2C_Add = PIC_Slave; //I2C Slave-Adresse vorbereiten
                I2C_Index_w = Count++; //Register/Daten die bertragen werden
                if (Count == 8) Count = 0; //Wieder von vorne
/***********************************************************************/                
                SEN = 1; //bertragung starten, lst SSP-Interrupt aus
/***********************************************************************/
            }
        }        
        if (m == 66) { //Zeitlich versetzt zu MLX_read()und LCD ausfhren
            if (n == 1) { //Werte vom 16F876A holen, alle 20ms im Wechsel mit T und LCD holen
                SSPADD = 4; //1MHz IC-Frequenz, 20MHz/(4*(4+1))=1MHz (PICTime)
                I2C_Count = 0; //Schrittzhler zurcksetzen
                I2C_Write = 0; //Es wird nicht geschrieben
                I2C_Read = 1; //Jetzt wird gelesen
                I2C_Add = PIC_Slave; //I2C Slave-Adresse vorbereiten
                I2C_Index_R = 7; //Datum wird unten rechts aufs LCD geschrieben
/************************************************************************/
                SEN = 1; //bertragung starten, lst SSP-Interrupt aus
/************************************************************************/
            }
        }
        TMR0IF = 0; //Schluss mit TMR0-Interrupt
//AD-Wandlung starten
        //Jetzt sollten die 5us vorbei sein
        //Analogwert initialisieren, alle 0,102ms
        GO_nDONE = 1; //Wandlung starten (Bit 2 von ADCON0, S. 127)
        //Die Wandlung braucht 40us, es dauert aber 100us bis der Wert beim 
        //nchsten TMR0-Interrupt ausgelesen wird.
//        Status = 0; //Zum Debuggen
    }
    //Interrupt durch I2C
    if (SSPIF) { //Hat den Interrupt das SSP-Modul ausgelst?
        SSPIF = 0; //Flag lschen, gell?
        if (I2C_Write) { //Soll geschrieben werden?
            switch (I2C_Count) { //Je nachdem welchen Wert I2C_Count hat
                //I2C_Count = 0, erster Interrupt nach SEN
                case 0:
                    SSPBUF = (I2C_Add << 1) & 0b11111110; //Slaveadresse = 20[0010100X],
                    //nach CLK<9> SSPIF = 1;            LSB=0-> Master schreibt
                    I2C_Count++; //Beim nchsten Interrupt case 1 bearbeiten
                    break; //Die Funktion switch() verlassen
                //I2C_Count = 1;
                case 1:
                    SSPBUF = Register_S[I2C_Index_w]; //Erstes Datum schreiben
                    I2C_Count++; //Beim nchsten Interrupt case 2 bearbeiten
                    break; //Die Funktion switch() verlassen
                //I2C_Count = 2;
                case 2:
                    SSPBUF = Datum_S[I2C_Index_w]; //Zweites Datum schreiben
                    I2C_Count++; //Beim nchsten Interrupt case 3 bearbeiten
                    break; //Die Funktion switch() verlassen
                //I2C_Count = 3;
                case 3:
                    PEN = 1; //bertragung beenden
                    I2C_Count++; //Brauchts eigentlich nicht, aber wer wei...
                    I2C_Write = 0; //Flag lschen
                    break;
                //I2C_Count > 3;
                default: //Bis hierher kommt das Programm normal nicht
                    break; //Die Funktion switch() verlassen
            }
        }
        //Es wird vom PIC 16F876A nur ein Datum gelesen
        if (I2C_Read) { //Lesen ist dran
            switch (I2C_Count) { //Je nachdem welchen Wert I2C_Count hat
                //I2C_Count = 0, erster Interrupt nach SEN
                case 0:
                    SSPBUF = (I2C_Add << 1) | 0b00000001; //LSB=1-> Master liest
                    I2C_Count++; //Beim nchsten Interrupt case 1 bearbeiten
                    break; //Die Funktion switch() verlassen
                //I2C_Count = 1;
                case 1:
                    RCEN = 1; //receive enable
                    I2C_Count++; //Beim nchsten Interrupt case 2 bearbeiten
                    break; //Die Funktion switch() verlassen
                //I2C_Count = 2;
                case 2:
                    Datum_R = SSPBUF; //Hier ist RCEN wieder 0
/****************************************************************/
//                    Datum_S[I2C_Index_R] = Datum_R; //Auf LCD anzeigen
/****************************************************************/
                    ACKDT = 1; //Not Acknowledge, beendet die read-Sequenz
                    ACKEN = 1; //Wert von ACKDT wird gesendet, 9. Flanke
                    I2C_Count++; //Beim nchsten Interrupt case 3 bearbeiten
                    break; //Die Funktion switch() verlassen
                    //I2C_Count = 3;
                case 3:
                    PEN = 1; //Master ist fertig
                    I2C_Count++; //Beim nchsten Interrupt default bearbeiten
                    I2C_Read = 0; //Flag lschen
                    break; //Die Funktion switch() verlassen
                    //I2C_Count >3;
                default: //Bis hierher kommt das Programm normal nicht
                    break; //Die Funktion switch() verlassen
            }
        }
//        //Nach PEN springt beim erneuten SSP-Interrupt das Programmsofort hier
//        //her, weil weder I2C_Write noch I2C_Read HI ist
    }
       //PORTB Interrupt, Interrupt on Change
    //Init() anpassen: PORTB
    if (IOCIF) {
        //IOCIF wird gelscht, wenn alle IOCBF gelscht sind!!!
        //IOCIF = 0; geht nicht!!!
        if (IOCBF0) { //Hat den Interrupt PORTB,0 ausgelst?
            Encoder++; //Encoder inkrementieren
            /*+++Achtung: Hier ganz speziell Ausgabe auf LCD Tab 18+++*/
            Datum_S [4] = Encoder; //
            IOCBF0 = 0; //Flag lschen
        }
        //
        if (Flag_Strecke_messen) { //Wird eine Strecke gefahren?
            if (Encoder >= Strecke) { //Ende der Strecke erreicht?
                Flag_Strecke_erreicht = 1; //sic
            }
        }
        
        if (IOCBF5) { //Hat den Interrupt PORTB,5 ausgelst?
            IOCBF5 = 0; //Flag lschen
            if (!TMR1IF) { //TMR1 ist nicht bergelaufen? Strecke < 2m (13107us Laufzeit)
                Zeit_US = TMR1H; //Timer 1 HI-Teil
                Abstand_US = Zeit_US - 9; //Abstand_US ~ tats. Abstand(ca. 10% Abweichung)
            }
            else {
                Abstand_US = 255; //Gemessene Strecke > 2m
                TMR1IF = 0;
            }
            Datum_S[5] = Abstand_US; //Anzeige auf LCD
        }
    }
//    Status = 0; //Hilfreich frs debuggen
}

